'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var runtimeCore = require('@vue/runtime-core');
var axios = _interopDefault(require('axios'));

function usePromise(fn, throwException = false) {
    if (!fn) {
        throw new Error(`[usePromise] argument can't be '${fn}'`);
    }
    if (typeof fn !== "function") {
        throw new Error(`[usePromise] expects function, but received ${typeof fn}`);
    }
    const loading = runtimeCore.ref(false);
    const error = runtimeCore.ref(null);
    const result = runtimeCore.ref(null);
    const promise = runtimeCore.ref();
    const exec = async (...args) => {
        loading.value = true;
        error.value = null;
        result.value = null;
        const currentPromise = (promise.value = fn(...args));
        try {
            const r = await currentPromise;
            if (promise.value === currentPromise) {
                result.value = r;
            }
            return r;
        }
        catch (er) {
            if (promise.value === currentPromise) {
                error.value = er;
                result.value = null;
            }
            return throwException ? currentPromise : undefined;
        }
        finally {
            if (promise.value === currentPromise) {
                loading.value = false;
            }
        }
    };
    return {
        exec,
        result,
        promise,
        loading,
        error
    };
}

/* istanbul ignore next  */
const _axios = axios || (globalThis && globalThis.axios);
// TODO replace by shared project
const isObject = (d) => typeof d === 'object';
const isString = (d) => typeof d === 'string';
const isBoolean = (d) => typeof d === 'boolean';
function useAxios(configUrl, configThrowException, throwException = false) {
    const config = !isString(configUrl) ? configUrl : isObject(configThrowException) ? configThrowException : undefined;
    throwException = isBoolean(configThrowException) ? configThrowException : throwException;
    const axiosClient = _axios.create(config);
    const client = runtimeCore.computed(() => axiosClient);
    const use = usePromise(async (request) => {
        return axiosClient.request(request);
    }, throwException);
    const data = runtimeCore.computed(() => (use.result.value && use.result.value.data) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.data) ||
        null);
    const status = runtimeCore.computed(() => (use.result.value && use.result.value.status) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.status) ||
        null);
    const statusText = runtimeCore.computed(() => (use.result.value && use.result.value.statusText) ||
        (use.error.value &&
            use.error.value.response &&
            use.error.value.response.statusText) ||
        null);
    // if url provided in the config, execute it straight away
    if (typeof configUrl === 'string') {
        use.exec({ url: configUrl });
    }
    else if (config && config.url) {
        use.exec(config);
    }
    return {
        ...use,
        client,
        data,
        status,
        statusText
    };
}

exports.useAxios = useAxios;
